/*************************************************************************
 * photonSim.cc                                                          *
 * Implementation file for the class photonSim                           *
 *                                                                       *
 * This file is part of the photonSim package                            *
 * Copyright 2020, Philipp Kampmann                                      *
 *                                                                       *
 * Licensed under GPL-3.0-or-later                                       *
 *                                                                       *
 *************************************************************************/

#include"photonSim.hh"
#include<iostream>
#include<TFile.h>
#include<TF1.h>
#include<TMath.h>
#include<TRandom.h>
#include<TGraph.h>

photonSim::photonSim()
  :m_e(0.51099895)
  ,alpha(0.0072973525693)
{

  tf1_pdf = new TF1("PDF Function",[&](double*x, double *p){ return pdf_theta(x,p); }, 0, TMath::Pi(), 2);
  probFile = new TFile("../data/procProbLAB.root");
  ComptonProb = (TGraph*) probFile->Get("Compton");
  PhotoEffProb = (TGraph*) probFile->Get("PhotoEff");
  RaylScatProb = (TGraph*) probFile->Get("RaylScat");

  ComptonProb->SetBit(TGraph::kIsSortedX);
  PhotoEffProb->SetBit(TGraph::kIsSortedX);
  RaylScatProb->SetBit(TGraph::kIsSortedX);
}

photonSim::~photonSim(){
  delete tf1_pdf;
  delete probFile;

}

double photonSim::pdf_theta(double* x, double* p){
  return TMath::Sin(x[0])*p[0]*kleinnishina(p[1],x[0])*2*TMath::Pi(); //*d_theta for integration, sin(x) = functional determinant
}

double photonSim::kleinnishina(double inipe, double theta){
  double ratio = gamma_energy(inipe,theta) / inipe;
  double retval=1.;
  retval*=1./2.*alpha*alpha/m_e/m_e;
  retval*=ratio*ratio;
  retval*=(ratio+1./ratio-TMath::Sin(theta)*TMath::Sin(theta));
  return retval;
}

double photonSim::gamma_energy(double inipe, double theta){
  return inipe/(1+inipe/m_e *(1-TMath::Cos(theta)));
}

unsigned int photonSim::decideProcess(const double& egamma){

  double myrand = gRandom->Uniform();
  unsigned int retval = -1;
  double photoeff = PhotoEffProb->Eval(egamma);

  if(egamma>0.99) return 0; //Always compton for high energies - pair production is neglected

  if(myrand<photoeff){
    retval = 1; //photo effect
  }
  else if(myrand<(photoeff+RaylScatProb->Eval(egamma))){
    retval = 2; //rayleigh scattering
  }
  else{
    retval = 0; // compton scattering
  }
  return retval;
}

void photonSim::simulatePhoton(double &egamma, double &eelectron){

  switch(decideProcess(egamma)){
    case 0: //Compton
      simulateCompton(egamma,eelectron);
      break;
    case 1: //PhotoEffect
      simulatePhotoEff(egamma,eelectron);
      break;
    default: //Rayleigh, start again
      simulatePhoton(egamma,eelectron);
  }
}

void photonSim::simulateCompton(double &egamma, double &eelectron){
  const double initegamma = egamma;
  tf1_pdf->SetParameter(0,1.);
  tf1_pdf->SetParameter(1,initegamma);
  const double tmptheta = tf1_pdf->GetRandom();
  egamma = gamma_energy(initegamma,tmptheta);
  eelectron = initegamma - egamma; 
}

void photonSim::simulatePhotoEff(double &egamma, double &eelectron){
  eelectron = egamma;
  egamma = 0.;
}
